@@ -1,4 +1,4 @@
-# Copyright (C) 2013 Canonical Ltd.
+# Copyright (C) 2013-2014 Canonical Ltd.
 # Author: Barry Warsaw <barry@ubuntu.com>
 
 # This program is free software: you can redistribute it and/or modify
@@ -18,6 +18,7 @@
 __all__ = [
     'TestDBusApply',
     'TestDBusCheckForUpdate',
+    'TestDBusCheckForUpdateException',
     'TestDBusClient',
     'TestDBusDownload',
     'TestDBusGetSet',
@@ -29,6 +30,7 @@ __all__ = [
     'TestDBusMockNoUpdate',
     'TestDBusMockUpdateAutoSuccess',
     'TestDBusMockUpdateManualSuccess',
+    'TestDBusMultipleChecksInFlight',
     'TestDBusPauseResume',
     'TestDBusProgress',
     'TestDBusRegressions',
@@ -48,7 +50,7 @@ from dbus.exceptions import DBusException
 from functools import partial
 from systemimage.bindings import DBusClient
 from systemimage.config import Configuration
-from systemimage.helpers import safe_remove
+from systemimage.helpers import atomic, safe_remove
 from systemimage.reactor import Reactor
 from systemimage.testing.helpers import (
     copy, find_dbus_process, make_http_server, setup_index, setup_keyring_txz,
@@ -122,6 +124,61 @@ class PausingReactor(Reactor):
         self.quit()
 
 
+class DoubleCheckingReactor(Reactor):
+    def __init__(self, iface):
+        super().__init__(dbus.SystemBus())
+        self.iface = iface
+        self.uas_signals = []
+        self.react_to('UpdateAvailableStatus')
+        self.react_to('UpdateDownloaded')
+        self.schedule(self.iface.CheckForUpdate)
+
+    def _do_UpdateAvailableStatus(self, signal, path, *args, **kws):
+        # We'll keep doing this until we get the UpdateDownloaded signal.
+        self.uas_signals.append(args)
+        self.schedule(self.iface.CheckForUpdate)
+
+    def _do_UpdateDownloaded(self, *args, **kws):
+        self.quit()
+
+
+class ManualUpdateReactor(Reactor):
+    def __init__(self, iface):
+        super().__init__(dbus.SystemBus())
+        self.iface = iface
+        self.rebooting = False
+        self.react_to('UpdateAvailableStatus')
+        self.react_to('UpdateProgress')
+        self.react_to('UpdateDownloaded')
+        self.react_to('Rebooting')
+        self.react_to('UpdateFailed')
+        self.iface.CheckForUpdate()
+
+    def _do_UpdateAvailableStatus(self, signal, path, *args, **kws):
+        # When the update is available, start the download.
+        self.iface.DownloadUpdate()
+
+    def _do_UpdateProgress(self, signal, path, *args, **kws):
+        # Once the download is in progress, initiate another check.  Only do
+        # this on the first progress signal.
+        if args == (0, 0):
+            self.iface.CheckForUpdate()
+
+    def _do_UpdateDownloaded(self, signal, path, *args, **kws):
+        # The update successfully downloaded, so apply the update now.
+        self.iface.ApplyUpdate()
+
+    def _do_UpdateFailed(self, signal, path, *args, **kws):
+        # Before LP: #1287919 was fixed, this signal would have been sent.
+        self.rebooting = False
+        self.quit()
+
+    def _do_Rebooting(self, signal, path, *args, **kws):
+        # The system is now rebooting <wink>.
+        self.rebooting = True
+        self.quit()
+
+
 class _TestBase(unittest.TestCase):
     """Base class for all DBus testing."""
 
@@ -262,13 +319,14 @@ class _LiveTesting(_TestBase):
         safe_remove(self.reboot_log)
         super().tearDown()
 
-    def _prepare_index(self, index_file):
+    def _prepare_index(self, index_file, write_callback=None):
         serverdir = SystemImagePlugin.controller.serverdir
         index_path = os.path.join(serverdir, 'stable', 'nexus7', 'index.json')
         head, tail = os.path.split(index_path)
         copy(index_file, head, tail)
         sign(index_path, 'device-signing.gpg')
-        setup_index(index_file, serverdir, 'device-signing.gpg')
+        setup_index(index_file, serverdir, 'device-signing.gpg',
+                    write_callback)
 
     def _touch_build(self, version):
         # Unlike the touch_build() helper, this one uses our own config object
@@ -374,6 +432,23 @@ class TestDBusCheckForUpdate(_LiveTesting):
         self.assertEqual(last_update_date, '2013-01-20 12:01:45')
         # All other values are undefined.
 
+    def test_check_for_update_twice(self):
+        # Issue two CheckForUpdate calls immediate after each other.
+        self.download_always()
+        reactor = SignalCapturingReactor('UpdateAvailableStatus')
+        def two_calls():
+            self.iface.CheckForUpdate()
+            self.iface.CheckForUpdate()
+        reactor.run(two_calls)
+        self.assertEqual(len(reactor.signals), 1)
+        # There's one boolean argument to the result.
+        (is_available, downloading, available_version, update_size,
+         last_update_date,
+         # descriptions,
+         error_reason) = reactor.signals[0]
+        self.assertTrue(is_available)
+        self.assertTrue(downloading)
+
     @unittest.skip('LP: #1215586')
     def test_get_multilingual_descriptions(self):
         # The descriptions are multilingual.
@@ -1368,7 +1443,9 @@ class TestDBusPauseResume(_LiveTesting):
     def setUp(self):
         super().setUp()
         # We have to hack the files to be rather large so that the download
-        # doesn't complete before we get a chance to pause it.
+        # doesn't complete before we get a chance to pause it.  Of course,
+        # this breaks the signatures because we're changing the file contents
+        # after the .asc files have been written.
         for path in ('3/4/5.txt', '4/5/6.txt', '5/6/7.txt'):
             full_path = os.path.join(
                 SystemImagePlugin.controller.serverdir, path)
@@ -1391,15 +1468,25 @@ class TestDBusPauseResume(_LiveTesting):
         self.assertTrue(reactor.paused)
         # Now let's resume the download.  Because we intentionally corrupted
         # the downloaded files, we'll get an UpdateFailed signal instead of
-        # the successful UpdateDownloaded signal.  We can ignore that.
+        # the successful UpdateDownloaded signal.
         reactor = SignalCapturingReactor('UpdateFailed')
         reactor.run(self.iface.DownloadUpdate, timeout=60)
         self.assertEqual(len(reactor.signals), 1)
-        # We've gotten one error and the first file that failed is 5.txt.
+        # The error message will include lots of details on the SignatureError
+        # that results.  The key thing is that it's 5.txt that is the first
+        # file to fail its signature check.
         failure_count, last_error = reactor.signals[0]
         self.assertEqual(failure_count, 1)
-        # Watch out for the trailing newline.
-        self.assertEqual(os.path.basename(last_error[:-1]), '5.txt')
+        check_next = False
+        for line in last_error.splitlines():
+            line = line.strip()
+            if check_next:
+                self.assertEqual(os.path.basename(line), '5.txt')
+                break
+            if line.startswith('data path:'):
+                check_next = True
+        else:
+            raise AssertionError('Did not find expected error output')
 
 
 class TestDBusUseCache(_LiveTesting):
@@ -1513,3 +1600,120 @@ update 7.txt 7.txt.asc
 update 5.txt 5.txt.asc
 unmount system
 """)
+
+
+class TestDBusMultipleChecksInFlight(_LiveTesting):
+    def test_multiple_check_for_updates(self):
+        # Log analysis of LP: #1277589 appears to show the following scenario,
+        # reproduced in this test case:
+        #
+        # * Automatic updates are enabled.
+        # * No image signing or image master keys are present.
+        # * A full update is checked.
+        #   - A new image master key and image signing key is downloaded.
+        #   - Update is available
+        #
+        # Start by creating some big files which will take a while to
+        # download.
+        def write_callback(dst):
+            # Write a 100 MiB sized file.
+            with open(dst, 'wb') as fp:
+                for i in range(25600):
+                    fp.write(b'x' * 4096)
+        self._prepare_index('index_24.json', write_callback)
+        # Create a reactor that will exit when the UpdateDownloaded signal is
+        # received.  We're going to issue a CheckForUpdate with automatic
+        # updates enabled.  As soon as we receive the UpdateAvailableStatus
+        # signal, we'll immediately issue *another* CheckForUpdate, which
+        # should run while the auto-download is working.
+        #
+        # As per LP: #1284217, we will get a second UpdateAvailableStatus
+        # signal, since the status is available even while the original
+        # request is being downloaded.
+        reactor = DoubleCheckingReactor(self.iface)
+        reactor.run()
+        # We need to have received at least 2 signals, but due to timing
+        # issues it could possibly be more.
+        self.assertGreater(len(reactor.uas_signals), 1)
+        # All received signals should have the same information.
+        for signal in reactor.uas_signals:
+            (is_available, downloading, available_version, update_size,
+             last_update_date,
+             #descriptions,
+             error_reason) = signal
+            self.assertTrue(is_available)
+            self.assertTrue(downloading)
+            self.assertEqual(available_version, '1600')
+            self.assertEqual(update_size, 314572800)
+            self.assertEqual(last_update_date, 'Unknown')
+            self.assertEqual(error_reason, '')
+
+    def test_multiple_check_for_updates_with_manual_downloading(self):
+        # Log analysis of LP: #1287919 (a refinement of LP: #1277589 with
+        # manual downloading enabled) shows that it's possible to enter the
+        # checking phase while a download of the data files is still running.
+        # When manually downloading, this will start another check, and as
+        # part of that check, the blacklist and other files will be deleted
+        # (in anticipation of them being re-downloaded).  When the data files
+        # are downloaded, the state machine that just did the data download
+        # may find its files deleted out from underneath it by the state
+        # machine doing the checking.
+        self.download_manually()
+        # Start by creating some big files which will take a while to
+        # download.
+        def write_callback(dst):
+            # Write a 100 MiB sized file.
+            with open(dst, 'wb') as fp:
+                for i in range(25600):
+                    fp.write(b'x' * 4096)
+        self._prepare_index('index_24.json', write_callback)
+        self._touch_build(0)
+        # Create a reactor that implements the following test plan:
+        # * Set the device to download manually.
+        # * Flash to an older revision
+        # * Open System Settings and wait for it to say Updates available
+        # * Click on About this phone
+        # * Click on Check for Update and wait for it to say Install 1 update
+        # * Click on Install 1 update and while the files are downloading,
+        #   swipe up from the bottom and click Back
+        # * Click on Check for Update again
+        # * Wait for the Update System overlay to come up, and then install
+        #   the update, and reboot
+        reactor = ManualUpdateReactor(self.iface)
+        reactor.run()
+        self.assertTrue(reactor.rebooting)
+
+
+class TestDBusCheckForUpdateException(_LiveTesting):
+    @classmethod
+    def setUpClass(cls):
+        ini_path = SystemImagePlugin.controller.ini_path
+        with open(ini_path, 'r', encoding='utf-8') as fp:
+            lines = fp.readlines()
+        with atomic(ini_path) as fp:
+            for line in lines:
+                key, sep, value = line.partition(': ')
+                if key == 'cache_partition':
+                    cls.bad_path = os.path.join(value.strip(), 'unwritable')
+                    print('{}: {}'.format(key, cls.bad_path), file=fp)
+                    os.makedirs(cls.bad_path, mode=0)
+                else:
+                    fp.write(line)
+        super().setUpClass()
+
+    def tearDown(self):
+        os.chmod(self.bad_path, 0o777)
+        super().tearDown()
+
+    def test_check_for_update_error(self):
+        # CheckForUpdate sees an error, in this case because the destination
+        # directory for downloads is not writable.  We'll get an
+        # UpdateAvailableStatus with an error string.
+        reactor = SignalCapturingReactor('UpdateAvailableStatus')
+        reactor.run(self.iface.CheckForUpdate)
+        self.assertEqual(len(reactor.signals), 1)
+        (is_available, downloading, available_version, update_size,
+         last_update_date,
+         # descriptions,
+         error_reason) = reactor.signals[0]
+        self.assertIn('Permission denied', error_reason)
