@@ -1,4 +1,4 @@
-# Copyright (C) 2013 Canonical Ltd.
+# Copyright (C) 2013-2014 Canonical Ltd.
 # Author: Barry Warsaw <barry@ubuntu.com>
 
 # This program is free software: you can redistribute it and/or modify
@@ -26,11 +26,13 @@ __all__ = [
 
 
 import os
+import sys
 import dbus
 import stat
 import time
 import shutil
 import unittest
+import subprocess
 
 from contextlib import ExitStack, contextmanager
 from datetime import datetime
@@ -41,7 +43,7 @@ from systemimage.config import Configuration, config
 from systemimage.helpers import safe_remove
 from systemimage.main import main as cli_main
 from systemimage.testing.helpers import (
-    ServerTestBase, configuration, copy, data_path, find_dbus_process,
+    ServerTestBase, chmod, configuration, copy, data_path, find_dbus_process,
     temporary_directory, touch_build)
 from systemimage.testing.nose import SystemImagePlugin
 from textwrap import dedent
@@ -484,6 +486,34 @@ class TestCLIMain(unittest.TestCase):
             last update: 2013-08-01 12:11:10
             """))
 
+    @configuration
+    def test_state_machine_exceptions(self, ini_file):
+        # If an exception happens during the state machine run, the error is
+        # logged and main exits with code 1.
+        config = Configuration()
+        config.load(ini_file)
+        self._resources.enter_context(
+            patch('systemimage.main.sys.argv', ['argv0', '-C', ini_file]))
+        # Making the cache directory unwritable is a good way to trigger a
+        # crash.  Be sure to set it back though!
+        with chmod(config.updater.cache_partition, 0):
+            exit_code = cli_main()
+        self.assertEqual(exit_code, 1)
+
+    @configuration
+    def test_state_machine_exceptions_dry_run(self, ini_file):
+        # Like above, but doing only a --dry-run.
+        config = Configuration()
+        config.load(ini_file)
+        # Making the cache directory unwritable is a good way to trigger a
+        # crash.  Be sure to set it back though!
+        self._resources.enter_context(
+            patch('systemimage.main.sys.argv',
+                  ['argv0', '-C', ini_file, '--dry-run']))
+        with chmod(config.updater.cache_partition, 0):
+            exit_code = cli_main()
+        self.assertEqual(exit_code, 1)
+
 
 class TestCLIMainDryRun(ServerTestBase):
     INDEX_FILE = 'index_14.json'
@@ -781,3 +811,25 @@ class TestDBusMain(unittest.TestCase):
         self.assertEqual(stat.filemode(mode), 'drwx--S---')
         mode = os.stat(config.system.logfile).st_mode
         self.assertEqual(stat.filemode(mode), '-rw-------')
+
+    def test_single_instance(self):
+        # Only one instance of the system-image-dbus service is allowed to
+        # remain active on a single system bus.
+        self.assertIsNone(find_dbus_process(self.ini_path))
+        self._activate()
+        proc = find_dbus_process(self.ini_path)
+        # Attempt to start a second process on the same system bus.
+        env = dict(
+            DBUS_SYSTEM_BUS_ADDRESS=os.environ['DBUS_SYSTEM_BUS_ADDRESS'])
+        args = (sys.executable, '-m', 'systemimage.service',
+                '-C', self.ini_path)
+        second = subprocess.Popen(args, universal_newlines=True, env=env)
+        # Allow a TimeoutExpired exception to fail the test.
+        try:
+            code = second.wait(timeout=10)
+        except subprocess.TimeoutExpired:
+            second.kill()
+            second.communicate()
+            raise
+        self.assertNotEqual(second.pid, proc)
+        self.assertEqual(code, 2)
